function makeWord(b1, b2)
	return b2 * 256 + b1;
end

function makeDword(b1, b2, b3, b4)
	return makeWord(b3, b4) * 65536 + makeWord(b1, b2);
end

bmpToSend = {}
bmpLineIdx = 1;

function bmp_timer()
	local line = bmpToSend[bmpLineIdx];
	sendln(line);

	if bmpLineIdx == #bmpToSend then
		unregister_timer("bmp_timer");
	else
		bmpLineIdx = bmpLineIdx + 1;
	end
end

function drawbmp(filename)
	local fd, err = io.open(filename, "rb");
	if fd == nil then
		println("Failed to open \"" .. filename .. "\": " .. err);
		return;
	end

	-- Read and check BITMAPFILEHEADER
	local bmpFileHeader = fd:read(14)
	if makeWord(bmpFileHeader:byte(1, 2)) ~= 0x4D42 then
		println("Not a BMP file (Bad header)");
		fd:close();
		return;
	elseif makeDword(bmpFileHeader:byte(3, 6)) > 64*1024 then
		println("File too large");
		fd:close();
		return;
	end

	-- Read and check BITMAPINFOHEADER
	local bmpInfoHeader = fd:read(40)
	local imageW = makeDword(bmpInfoHeader:byte(5, 8));
	local imageH = makeDword(bmpInfoHeader:byte(9, 12));
	local stride = 4 * math.floor((imageW * 3 + 3) / 4);
	if makeWord(bmpInfoHeader:byte(13, 14)) ~= 1 or makeWord(bmpInfoHeader:byte(15, 16)) ~= 24 then
		println("File is not 24-bit");
		fd:close();
		return;
	end
	if makeDword(bmpInfoHeader:byte(17, 20)) ~= 0 then
		println("File is not uncompressed");
		fd:close();
		return;
	end

	-- Check dimensions
	if imageW > 80 or imageH > 80 then
		println("File is too large - max is 80x80 (Or you'll just get kicked)");
		fd:close();
		return;
	end

	-- Process scanlines
	local imageStart = makeDword(bmpFileHeader:byte(11, 14));
	bmpToSend = {}
	bmpLineIdx = 1;
	for y=1, imageH do
		fd:seek("set", imageStart + (imageH-y) * stride);
		local line = "<-b>";
		for x=1, imageW do
			local rgb = fd:read(3);
			local r = rgb:byte(3,3);
			local g = rgb:byte(2,2);
			local b = rgb:byte(1,1);
			if r < 0x40 and g < 0x40 and b < 0xa4 then
				if r < 0x40 then r = 0x40 end
				if g < 0x40 then g = 0x40 end
				if b < 0x40 then b = 0x40 end
			end
			line = line .. string.format("<-c%02x%02x%02x>#", r, g, b);
		end
		bmpToSend[y] = line;
	end
	fd:close();

	if imageW > 40 then
		register_timer(250, "bmp_timer");
	else
		register_timer(100, "bmp_timer");
	end
end
register_command("bmp", "drawbmp");
