-- Globals
g_pingRunning = false;
g_pingKey = nil;
g_pingTime = 0;

-- Format ping message
function sendPingReply(time)
	sendln(string.format("I have a ping of <-1><-36>.:<-32>%dms<-36>:.", time));
end

-- Generates a 24 character unique key to send as .ping param
function generatePingKey()
	local key = "";
	for i = 1, 24, 1 do
		key = key .. math.floor(math.random() * 10);
	end

	return key;
end

-- Reset everything for a clean ping
function cleanupPing()

	-- Reset globals
	g_pingRunning = false;
	g_pingKey = nil;
	g_pingTime = 0;

	-- Unregister events
	unregister_event("onReceivePing");
	unregister_timer("onPingTimeout");

	-- Come out of high performance mode
	exitHighPerfMode();
end

-- Handler for ping replies
function onReceivePing(args)
	if (g_pingRunning and string.sub(args, 0, string.len("Pong!")) == "Pong!" and
		g_pingKey == string.sub(args, string.find(args, ' ') + 1, -3)) then

		-- Send reply
		sendPingReply(getTime("u") - g_pingTime);
		cleanupPing();
		return "";
	end

	return args;
end

-- Ping timeout
function onPingTimeout()
	println("<-cff0000>Ping timed out!");
	cleanupPing();
end

-- Start a new ping
function cmdPing(args)

	-- Do nothing if we're already processing a ping
	if(g_pingRunning) then
		println("<-cff0000>Ping already in progress");
		return;
	end

	-- Do nothing if we're not connected
	if(isConnected() == 0) then
		println("<-cff0000>Can't ping; not connected to a server");
		return;
	end

	-- Go to high performance mode and setup ping info
	enterHighPerfMode();
	g_pingRunning = true;
	g_pingKey = generatePingKey();
	g_pingTime = getTime("u");

	-- Set timeout timer

	-- Send ping, hook onReceive and setup timeout timer
	sendln(".ping " .. g_pingKey);
	register_event("onReceive", "onReceivePing");
	register_timer(10000, "onPingTimeout");
end

register_command("ping", "cmdPing");
